import { Is } from "./is";
export var DomElement;
(function (DomElement) {
    function createWithNoContainer(type) {
        const nodeType = type.toLowerCase();
        const isText = nodeType === "text";
        let result = isText ? document.createTextNode("") : document.createElement(nodeType);
        return result;
    }
    DomElement.createWithNoContainer = createWithNoContainer;
    function create(container, type, className = "", beforeNode = null) {
        const nodeType = type.toLowerCase();
        const isText = nodeType === "text";
        let result = isText ? document.createTextNode("") : document.createElement(nodeType);
        if (Is.defined(className)) {
            result.className = className;
        }
        if (Is.defined(beforeNode)) {
            container.insertBefore(result, beforeNode);
        }
        else {
            container.appendChild(result);
        }
        return result;
    }
    DomElement.create = create;
    function createWithHTML(container, type, className, html, beforeNode = null) {
        const element = create(container, type, className, beforeNode);
        element.innerHTML = html;
        return element;
    }
    DomElement.createWithHTML = createWithHTML;
    function getStyleValueByName(element, stylePropertyName, toNumber = false) {
        let value = null;
        if (document.defaultView.getComputedStyle) {
            value = document.defaultView.getComputedStyle(element, null).getPropertyValue(stylePropertyName);
        }
        else if (element.currentStyle) {
            value = element.currentStyle[stylePropertyName];
        }
        if (toNumber) {
            value = parseFloat(value);
        }
        return value;
    }
    DomElement.getStyleValueByName = getStyleValueByName;
    function addClass(element, className) {
        element.className += " " + className;
        element.className = element.className.trim();
    }
    DomElement.addClass = addClass;
    function removeClass(element, className) {
        element.className = element.className.replace(className, "");
        element.className = element.className.trim();
    }
    DomElement.removeClass = removeClass;
    function cancelBubble(e) {
        e.preventDefault();
        e.cancelBubble = true;
    }
    DomElement.cancelBubble = cancelBubble;
    function getScrollPosition() {
        const doc = document.documentElement;
        const left = (window.pageXOffset || doc.scrollLeft) - (doc.clientLeft || 0);
        const top = (window.pageYOffset || doc.scrollTop) - (doc.clientTop || 0);
        return {
            left: left,
            top: top
        };
    }
    DomElement.getScrollPosition = getScrollPosition;
    function showElementAtMousePosition(e, element) {
        let left = e.pageX;
        let top = e.pageY;
        const scrollPosition = getScrollPosition();
        element.style.display = "block";
        if (left + element.offsetWidth > window.innerWidth) {
            left -= element.offsetWidth;
        }
        else {
            left++;
        }
        if (top + element.offsetHeight > window.innerHeight) {
            top -= element.offsetHeight;
        }
        else {
            top++;
        }
        if (left < scrollPosition.left) {
            left = e.pageX + 1;
        }
        if (top < scrollPosition.top) {
            top = e.pageY + 1;
        }
        element.style.left = left + "px";
        element.style.top = top + "px";
    }
    DomElement.showElementAtMousePosition = showElementAtMousePosition;
    function reverseChildrenOrder(parent) {
        const children = parent.children;
        let childrenLength = children.length - 1;
        for (; childrenLength--;) {
            parent.appendChild(children[childrenLength]);
        }
    }
    DomElement.reverseChildrenOrder = reverseChildrenOrder;
    function createCheckBox(container, labelText, checked = null, onClick = null) {
        const lineContainer = create(container, "div");
        const label = create(lineContainer, "label", "checkbox");
        const input = create(label, "input");
        input.type = "checkbox";
        if (Is.defined(onClick)) {
            input.onclick = onClick;
        }
        if (Is.defined(checked)) {
            input.checked = checked;
        }
        create(label, "span", "check-mark");
        createWithHTML(label, "span", "text", labelText);
        return {
            input: input,
            label: label
        };
    }
    DomElement.createCheckBox = createCheckBox;
})(DomElement || (DomElement = {}));
//# sourceMappingURL=dom.js.map